@@ -27,6 +27,13 @@ Weight <- setRefClass("Weight",
     initialize = function(o_gda_w) {
       "Constructor with a GeoDaWeight object (internally used)"
       .self$gda_w = o_gda_w
+      .self$Update(FALSE)
+    },
+    Update = function(updateStats = TRUE) {
+      "Update the weights meta data"
+      if (updateStats == TRUE) {
+        gda_w$GetNbrStats()
+      }
       .self$is_symmetric = gda_w$IsSymmetric()
       .self$sparsity = gda_w$GetSparsity()
       .self$min_neighbors = gda_w$GetMinNeighbors()
@@ -36,6 +43,14 @@ Weight <- setRefClass("Weight",
       .self$num_obs = gda_w$GetNumObs()
       .self$has_isolates = gda_w$HasIsolates()
     },
+    SetNeighbors = function(idx, nbrs) {
+      "Set neighbors for one observation"
+      gda_w$SetNeighbors(idx, nbrs)
+    },
+    SetNeighborsAndWeights = function(idx, nbrs, nbr_w) {
+      "Set neighbors with weights values for one observation"
+      gda_w$SetNeighborsAndWeights(idx, nbrs, nbr_w)
+    },
     IsSymmetric = function() {
       "Check if weights matrix is symmetric"
       return(gda_w$IsSymmetric())
@@ -117,6 +132,66 @@ summary.Weight <- function(object, ...) {
   format(output)
 }
 
+#################################################################
+#' @title Create an empty weights
+#' @description Create an empty weights
+#' @param num_obs The number of observations for this empty weights
+#' @return An instance of Weight-class
+#' @export
+create_weights <- function(num_obs) {
+  return (Weight$new(p_GeoDaWeight(num_obs)))
+}
+
+#################################################################
+#' @title Set neighbors of an observation
+#' @description Set neighbors for idx-th observation, idx starts from 1
+#' @param gda_w A Weight object
+#' @param idx A value indicates idx-th observation, idx start from 1
+#' @param nbrs A list indicates the neighbors of idx-th observation (id start from 1)
+#' @examples
+#' \dontrun{
+#' new_w <- create_weights(10)
+#' set_neighbors(new_w, 1, c(2,3))
+#' update_weights(new_w)
+#' }
+#' @export
+set_neighbors <- function(gda_w, idx, nbrs) {
+  gda_w$SetNeighbors(idx, nbrs)
+}
+
+#################################################################
+#' @title Set neighbors and weights values of an observation
+#' @description Set neighbors and the associated weights values for idx-th observation, idx starts from 1
+#' @param gda_w A Weight object
+#' @param idx A value indicates idx-th observation, idx start from 1
+#' @param nbrs A list indicates the neighbors of idx-th observation (id start from 1)
+#' @param wvals A list indicates the associated weights values of the neighbors
+#' @examples
+#' \dontrun{
+#' new_w <- create_weights(10)
+#' set_neighbors(new_w, 1, c(2,3))
+#' update_weights(new_w)
+#' }
+#' @export
+set_neighbors_with_weights <- function(gda_w, idx, nbrs, wvals) {
+  gda_w$SetNeighborsAndWeights(idx, nbrs, wvals)
+}
+
+#################################################################
+#' @title Update meta data of a spatial weights
+#' @description Update meta data of a spatial weights. This function can be used after calling `set_neighbor()` function .
+#' @param gda_w A Weight object
+#' @examples
+#' \dontrun{
+#' new_w <- create_weights(10)
+#' set_neighbors(new_w, 1, c(2,3))
+#' update_weights(new_w)
+#' }
+#' @export
+update_weights <- function(gda_w) {
+  gda_w$Update()
+}
+
 #################################################################
 #' @title Symmetry of Weights Matrix
 #' @description Check if weights matrix is symmetric
@@ -169,7 +244,7 @@ weights_sparsity <- function(gda_w) {
 }
 
 #################################################################
-#' @title Neighbor Information of Spatial Weights
+#' @title Neighbors of one observation
 #' @description Get neighbors for idx-th observation, idx starts from 1
 #' @param gda_w A Weight object
 #' @param idx A value indicates idx-th observation, idx start from 1
@@ -194,6 +269,32 @@ get_neighbors <- function(gda_w, idx) {
   return(rtn_nbrs)
 }
 
+#################################################################
+#' @title Weights values of the neighbors of one observation
+#' @description Get the associated weights values of neighbors for idx-th observation
+#' @param gda_w A Weight object
+#' @param idx A value indicates idx-th observation, idx start from 1
+#' @return A numeric vector of the neighbor indices, which start from 1
+#' @examples
+#' \dontrun{
+#' guerry_path <- system.file("extdata", "Guerry.shp", package = "rgeoda")
+#' guerry <- geoda_open(guerry_path)
+#' queen_w <- queen_weights(guerry)
+#' nbrs <- get_neighbors_weights(queen_w, idx = 1)
+#' cat("\nNeighbors of the 1-st observation are:", nbrs)
+#' }
+#' @export
+get_neighbors_weights <- function(gda_w, idx) {
+  idx <- idx - 1
+  nn <- gda_w$GetNeighborSize(idx)
+  nbrs <- gda_w$GetNeighborWeights(idx)
+  rtn_nbrs <- vector()
+  for (i in 1:nn) {
+    rtn_nbrs[i]  <- nbrs[i]
+  }
+  return(rtn_nbrs)
+}
+
 #################################################################
 #' @title Spatial Lag
 #' @description Compute the spatial lag for idx-th observation using selected variable and current weights matrix
@@ -533,7 +634,6 @@ gda_queen_weights <- function(geoda_obj, order=1, include_lower_order = FALSE, p
   return(Weight$new(p_GeoDaWeight(w)))
 }
 
-
 #################################################################
 #' @title (For internally use and test only) Rook Contiguity Spatial Weights
 #' @description Create a Rook contiguity weights with options of "order", "include lower order" and "precision threshold"
@@ -733,3 +833,102 @@ gda_kernel_knn_weights <- function(geoda_obj, k, kernel_method, adaptive_bandwid
 
   return(Weight$new(p_GeoDaWeight(w)))
 }
+
+#################################################################
+#' @title as.matrix
+#' @description Convert a spatial weights object to a Matrix object
+#' @param obj A weights object
+#' @return A matrix object
+#' @export
+as.matrix<- function(obj) {
+  if (length(class(obj)) == 1 && class(obj) == "Weight") {
+    n <- obj$num_obs
+    m <- matrix(0, nrow = n, ncol = n)
+
+    for (id in 1:n) {
+      nn <- obj$GetNeighborSize(id-1)
+      nbrs <- obj$GetNeighbors(id-1)
+      nbrWeights <- obj$GetNeighborWeights(id-1)
+
+      for (i in 1:nn) {
+        nid <- nbrs[i] + 1
+        wv <- nbrWeights[i]
+        m[id, nid] <- wv
+      }
+    }
+
+    return(m)
+  }
+}
+
+#################################################################
+#' @title Read a .GAL file
+#' @description Create a spatial weights object from a .GAL file
+#' @param file_path The file paht of the .GAL file
+#' @param id_vec The id_vec is the id values used in the .GAL file. Default is empty.
+#' @return A weights object
+#' @export
+read_gal <- function(file_path, id_vec = c()) {
+
+  # read first line from file
+  con <- file(file_path,"r")
+  first_line <- readLines(con,n=1)
+  close(con)
+
+  items <- strsplit(first_line, " ")
+  num_obs <- as.numeric(items[[1]][[2]])
+
+  if (length(id_vec) == 0) {
+    id_vec <- 0 : num_obs - 1
+  }
+
+  if (class(id_vec) == "numeric"){
+    id_vec <- as.character(id_vec)
+  }
+
+  w <- p_gda_load_gal(file_path, id_vec)
+
+  return(Weight$new(p_GeoDaWeight(w)))
+}
+
+#################################################################
+#' @title Read a .GWT file
+#' @description Create a spatial weights object from a .GWT file
+#' @param file_path The file paht of the .GWT file
+#' @param id_vec The id_vec is the id values used in the .GWT file. Default is empty.
+#' @return A weights object
+#' @export
+read_gwt <- function(file_path, id_vec = c()) {
+  # read first line from file
+  con <- file(file_path,"r")
+  first_line <- readLines(con,n=1)
+  close(con)
+
+  items <- strsplit(first_line, " ")
+  num_obs <- as.numeric(items[[1]][[2]])
+
+  if (length(id_vec) == 0) {
+    id_vec <- 0 : num_obs - 1
+  }
+
+  if (class(id_vec) == "numeric"){
+    id_vec <- as.character(id_vec)
+  }
+
+  w <- p_gda_load_gwt(file_path, id_vec)
+
+  return(Weight$new(p_GeoDaWeight(w)))
+}
+
+#################################################################
+#' @title Read a .SWM file
+#' @description Create a spatial weights object from a .SWM file
+#' @param file_path The file paht of the .SWM file
+#' @param id_vec The id_vec is the id values used in the .SWM file. e.g. c(0,1,2,3,...)
+#' @return A weights object
+#' @export
+read_swm <- function(file_path, id_vec = numeric()) {
+  w <- p_gda_load_swm(file_path, id_vec)
+
+  return(Weight$new(p_GeoDaWeight(w)))
+}
